@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
+# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
 #
+
 import re
 import sys
 import uuid
 from functools import cached_property
-from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple
 
 from snowflake.snowpark._internal.analyzer.table_function import GeneratorTableFunction
 
@@ -74,6 +75,14 @@ from snowflake.snowpark._internal.utils import (
 from snowflake.snowpark.row import Row
 from snowflake.snowpark.types import StructType
 
+# Python 3.8 needs to use typing.Iterable because collections.abc.Iterable is not subscriptable
+# Python 3.9 can use both
+# Python 3.10 needs to use collections.abc.Iterable because typing.Iterable is removed
+try:
+    from typing import Iterable
+except ImportError:
+    from collections.abc import Iterable
+
 
 class SnowflakePlan(LogicalPlan):
     class Decorator:
@@ -88,10 +97,13 @@ class SnowflakePlan(LogicalPlan):
                 try:
                     return func(*args, **kwargs)
                 except snowflake.connector.errors.ProgrammingError as e:
+                    query = None
+                    if "query" in e.__dict__:
+                        query = e.__getattribute__("query")
                     tb = sys.exc_info()[2]
                     if "unexpected 'as'" in e.msg.lower():
-                        ne = (
-                            SnowparkClientExceptionMessages.SQL_PYTHON_REPORT_UNEXPECTED_ALIAS()
+                        ne = SnowparkClientExceptionMessages.SQL_PYTHON_REPORT_UNEXPECTED_ALIAS(
+                            query
                         )
                         raise ne.with_traceback(tb) from None
                     elif e.sqlstate == "42000" and "invalid identifier" in e.msg:
@@ -124,7 +136,7 @@ class SnowflakePlan(LogicalPlan):
                                 unaliased_cols[0] if unaliased_cols else "<colname>"
                             )
                             ne = SnowparkClientExceptionMessages.SQL_PYTHON_REPORT_INVALID_ID(
-                                orig_col_name
+                                orig_col_name, query
                             )
                             raise ne.with_traceback(tb) from None
                         elif (
@@ -141,7 +153,7 @@ class SnowflakePlan(LogicalPlan):
                             > 1
                         ):
                             ne = SnowparkClientExceptionMessages.SQL_PYTHON_REPORT_JOIN_AMBIGUOUS(
-                                col, col
+                                col, col, query
                             )
                             raise ne.with_traceback(tb) from None
                         else:
@@ -485,9 +497,12 @@ class SnowflakePlanBuilder:
         join_type: JoinType,
         condition: str,
         source_plan: Optional[LogicalPlan],
+        use_constant_subquery_alias: bool,
     ):
         return self.build_binary(
-            lambda x, y: join_statement(x, y, join_type, condition),
+            lambda x, y: join_statement(
+                x, y, join_type, condition, use_constant_subquery_alias
+            ),
             left,
             right,
             source_plan,
