@@ -1,7 +1,8 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
+# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
 #
+
 import functools
 import os
 import sys
@@ -9,9 +10,8 @@ import time
 from logging import getLogger
 from typing import IO, Any, Dict, Iterator, List, Optional, Set, Tuple, Union
 
-import snowflake.connector
 from snowflake.connector import SnowflakeConnection, connect
-from snowflake.connector.constants import FIELD_ID_TO_NAME
+from snowflake.connector.constants import ENV_VAR_PARTNER, FIELD_ID_TO_NAME
 from snowflake.connector.cursor import ResultMetadata, SnowflakeCursor
 from snowflake.connector.errors import NotSupportedError, ProgrammingError
 from snowflake.connector.network import ReauthenticationRequest
@@ -47,9 +47,6 @@ from snowflake.snowpark.row import Row
 
 logger = getLogger(__name__)
 
-# set `paramstyle` to qmark for batch insertion
-snowflake.connector.paramstyle = "qmark"
-
 # parameters needed for usage tracking
 PARAM_APPLICATION = "application"
 PARAM_INTERNAL_APPLICATION_NAME = "internal_application_name"
@@ -147,7 +144,15 @@ class ServerConnection:
 
     def _add_application_name(self) -> None:
         if PARAM_APPLICATION not in self._lower_case_parameters:
-            self._lower_case_parameters[PARAM_APPLICATION] = get_application_name()
+            # Mirrored from snowflake-connector-python/src/snowflake/connector/connection.py#L295
+            if ENV_VAR_PARTNER in os.environ.keys():
+                self._lower_case_parameters[PARAM_APPLICATION] = os.environ[
+                    ENV_VAR_PARTNER
+                ]
+            elif "streamlit" in sys.modules:
+                self._lower_case_parameters[PARAM_APPLICATION] = "streamlit"
+            else:
+                self._lower_case_parameters[PARAM_APPLICATION] = get_application_name()
         if PARAM_INTERNAL_APPLICATION_NAME not in self._lower_case_parameters:
             self._lower_case_parameters[
                 PARAM_INTERNAL_APPLICATION_NAME
@@ -305,6 +310,8 @@ class ServerConnection:
         async_job_plan: Optional[
             SnowflakePlan
         ] = None,  # this argument is currently only used by AsyncJob
+        log_on_exception: bool = False,
+        case_sensitive: bool = True,
         **kwargs,
     ) -> Union[Dict[str, Any], AsyncJob]:
         try:
@@ -328,8 +335,9 @@ class ServerConnection:
                     f"Execute async query [queryID: {results_cursor['queryId']}] {query}"
                 )
         except Exception as ex:
-            query_id_log = f" [queryID: {ex.sfqid}]" if hasattr(ex, "sfqid") else ""
-            logger.error(f"Failed to execute query{query_id_log} {query}\n{ex}")
+            if log_on_exception:
+                query_id_log = f" [queryID: {ex.sfqid}]" if hasattr(ex, "sfqid") else ""
+                logger.error(f"Failed to execute query{query_id_log} {query}\n{ex}")
             raise ex
 
         # fetch_pandas_all/batches() only works for SELECT statements
@@ -348,6 +356,8 @@ class ServerConnection:
                 async_job_plan.session,
                 data_type,
                 async_job_plan.post_actions,
+                log_on_exception,
+                case_sensitive=case_sensitive,
                 **kwargs,
             )
 
@@ -395,6 +405,8 @@ class ServerConnection:
         to_iter: bool = False,
         block: bool = True,
         data_type: _AsyncResultType = _AsyncResultType.ROW,
+        log_on_exception: bool = False,
+        case_sensitive: bool = True,
         **kwargs,
     ) -> Union[
         List[Row], "pandas.DataFrame", Iterator[Row], Iterator["pandas.DataFrame"]
@@ -404,7 +416,14 @@ class ServerConnection:
                 "Async query is not supported in stored procedure yet"
             )
         result_set, result_meta = self.get_result_set(
-            plan, to_pandas, to_iter, **kwargs, block=block, data_type=data_type
+            plan,
+            to_pandas,
+            to_iter,
+            **kwargs,
+            block=block,
+            data_type=data_type,
+            log_on_exception=log_on_exception,
+            case_sensitive=case_sensitive,
         )
         if not block:
             return result_set
@@ -412,9 +431,13 @@ class ServerConnection:
             return result_set["data"]
         else:
             if to_iter:
-                return result_set_to_iter(result_set["data"], result_meta)
+                return result_set_to_iter(
+                    result_set["data"], result_meta, case_sensitive=case_sensitive
+                )
             else:
-                return result_set_to_rows(result_set["data"], result_meta)
+                return result_set_to_rows(
+                    result_set["data"], result_meta, case_sensitive=case_sensitive
+                )
 
     @SnowflakePlan.Decorator.wrap_exception
     def get_result_set(
@@ -424,6 +447,8 @@ class ServerConnection:
         to_iter: bool = False,
         block: bool = True,
         data_type: _AsyncResultType = _AsyncResultType.ROW,
+        log_on_exception: bool = False,
+        case_sensitive: bool = True,
         **kwargs,
     ) -> Tuple[
         Dict[
@@ -474,6 +499,8 @@ $$"""
                     block=block,
                     data_type=data_type,
                     async_job_plan=plan,
+                    log_on_exception=log_on_exception,
+                    case_sensitive=case_sensitive,
                     **kwargs,
                 )
 
@@ -499,6 +526,8 @@ $$"""
                             block=not is_last,
                             data_type=data_type,
                             async_job_plan=plan,
+                            log_on_exception=log_on_exception,
+                            case_sensitive=case_sensitive,
                             **kwargs,
                         )
                         placeholders[query.query_id_place_holder] = (
@@ -515,6 +544,8 @@ $$"""
                         action.sql,
                         is_ddl_on_temp_object=action.is_ddl_on_temp_object,
                         block=block,
+                        log_on_exception=log_on_exception,
+                        case_sensitive=case_sensitive,
                         **kwargs,
                     )
 
