@@ -1,7 +1,8 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
+# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
 #
+
 import array
 import contextlib
 import datetime
@@ -513,31 +514,40 @@ def column_to_bool(col_):
 
 
 def result_set_to_rows(
-    result_set: List[Any], result_meta: Optional[List[ResultMetadata]] = None
+    result_set: List[Any],
+    result_meta: Optional[List[ResultMetadata]] = None,
+    case_sensitive: bool = True,
 ) -> List[Row]:
     col_names = [col.name for col in result_meta] if result_meta else None
     rows = []
+    row_struct = Row
+    if col_names:
+        row_struct = (
+            Row._builder.build(*col_names).set_case_sensitive(case_sensitive).to_row()
+        )
     for data in result_set:
         if data is None:
             raise ValueError("Result returned from Python connector is None")
-        row = Row(*data)
-        # row might have duplicated column names
-        if col_names:
-            row._fields = col_names
+        row = row_struct(*data)
         rows.append(row)
     return rows
 
 
 def result_set_to_iter(
-    result_set: SnowflakeCursor, result_meta: Optional[List[ResultMetadata]] = None
+    result_set: SnowflakeCursor,
+    result_meta: Optional[List[ResultMetadata]] = None,
+    case_sensitive: bool = True,
 ) -> Iterator[Row]:
     col_names = [col.name for col in result_meta] if result_meta else None
+    row_struct = Row
+    if col_names:
+        row_struct = (
+            Row._builder.build(*col_names).set_case_sensitive(case_sensitive).to_row()
+        )
     for data in result_set:
         if data is None:
             raise ValueError("Result returned from Python connector is None")
-        row = Row(*data)
-        if col_names:
-            row._fields = col_names
+        row = row_struct(*data)
         yield row
 
 
