@@ -1,7 +1,8 @@
 #!/usr/bin/env python3
 #
-# Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
+# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
 #
+
 import datetime
 import decimal
 import json
@@ -12,7 +13,7 @@ from functools import reduce
 from logging import getLogger
 from threading import RLock
 from types import ModuleType
-from typing import Any, Dict, Iterable, List, Literal, Optional, Set, Tuple, Union
+from typing import Any, Dict, List, Literal, Optional, Set, Tuple, Union
 
 import cloudpickle
 import pkg_resources
@@ -25,7 +26,7 @@ from snowflake.snowpark._internal.analyzer.analyzer_utils import (
     escape_quotes,
     quote_name,
 )
-from snowflake.snowpark._internal.analyzer.datatype_mapper import str_to_sql, to_sql
+from snowflake.snowpark._internal.analyzer.datatype_mapper import str_to_sql
 from snowflake.snowpark._internal.analyzer.expression import Attribute
 from snowflake.snowpark._internal.analyzer.select_statement import (
     SelectSnowflakePlan,
@@ -49,9 +50,9 @@ from snowflake.snowpark._internal.telemetry import set_api_call_source
 from snowflake.snowpark._internal.type_utils import (
     ColumnOrName,
     infer_schema,
-    infer_type,
     merge_type,
 )
+from snowflake.snowpark._internal.udf_utils import generate_call_python_sp_sql
 from snowflake.snowpark._internal.utils import (
     MODULE_NAME_TO_PACKAGE_NAME_MAP,
     STAGE_PREFIX,
@@ -122,6 +123,14 @@ from snowflake.snowpark.types import (
 from snowflake.snowpark.udf import UDFRegistration
 from snowflake.snowpark.udtf import UDTFRegistration
 
+# Python 3.8 needs to use typing.Iterable because collections.abc.Iterable is not subscriptable
+# Python 3.9 can use both
+# Python 3.10 needs to use collections.abc.Iterable because typing.Iterable is removed
+try:
+    from typing import Iterable
+except ImportError:
+    from collections.abc import Iterable
+
 _logger = getLogger(__name__)
 
 _session_management_lock = RLock()
@@ -130,6 +139,7 @@ _PYTHON_SNOWPARK_USE_SCOPED_TEMP_OBJECTS_STRING = (
     "PYTHON_SNOWPARK_USE_SCOPED_TEMP_OBJECTS"
 )
 _PYTHON_SNOWPARK_USE_SQL_SIMPLIFIER_STRING = "PYTHON_SNOWPARK_USE_SQL_SIMPLIFIER"
+WRITE_PANDAS_CHUNK_SIZE: int = 100000 if is_in_stored_procedure() else None
 
 
 def _get_active_session() -> Optional["Session"]:
@@ -183,7 +193,7 @@ class Session:
 
     To create a :class:`Session` object from an existing Python Connector connection::
 
-        >>> session = Session.builder.configs(connection=<your python connector connection>).create() # doctest: +SKIP
+        >>> session = Session.builder.configs({"connection": <your python connector connection>}).create() # doctest: +SKIP
 
     :class:`Session` contains functions to construct a :class:`DataFrame` like :meth:`table`,
     :meth:`sql` and :attr:`read`, etc.
@@ -191,6 +201,45 @@ class Session:
     A :class:`Session` object is not thread-safe.
     """
 
+    class RuntimeConfig:
+        def __init__(self, session: "Session", conf: Dict[str, Any]) -> None:
+            self._session = session
+            self._conf = {
+                "use_constant_subquery_alias": True
+            }  # For config that's temporary/to be removed soon
+            for key, val in conf.items():
+                if self.is_mutable(key):
+                    self.set(key, val)
+
+        def get(self, key: str, default=None) -> Any:
+            if hasattr(Session, key):
+                return getattr(self._session, key)
+            if hasattr(self._session._conn._conn, key):
+                return getattr(self._session._conn._conn, key)
+            return self._conf.get(key, default)
+
+        def is_mutable(self, key: str) -> bool:
+            if hasattr(Session, key) and isinstance(getattr(Session, key), property):
+                return getattr(Session, key).fset is not None
+            if hasattr(SnowflakeConnection, key) and isinstance(
+                getattr(SnowflakeConnection, key), property
+            ):
+                return getattr(SnowflakeConnection, key).fset is not None
+            return key in self._conf
+
+        def set(self, key: str, value: Any) -> None:
+            if self.is_mutable(key):
+                if hasattr(Session, key):
+                    setattr(self._session, key, value)
+                if hasattr(SnowflakeConnection, key):
+                    setattr(self._session._conn._conn, key, value)
+                if key in self._conf:
+                    self._conf[key] = value
+            else:
+                raise AttributeError(
+                    f'Configuration "{key}" does not exist or is not mutable in runtime'
+                )
+
     class SessionBuilder:
         """
         Provides methods to set connection parameters and create a :class:`Session`.
@@ -227,16 +276,20 @@ class Session:
 
         def create(self) -> "Session":
             """Creates a new Session."""
-            if "connection" in self._options:
-                return self._create_internal(self._options["connection"])
-            return self._create_internal(conn=None)
+            session = self._create_internal(self._options.get("connection"))
+            return session
 
         def _create_internal(
             self, conn: Optional[SnowflakeConnection] = None
         ) -> "Session":
+            # Set paramstyle to qmark by default to be consistent with previous behavior
+            if "paramstyle" not in self._options:
+                self._options["paramstyle"] = "qmark"
             new_session = Session(
-                ServerConnection({}, conn) if conn else ServerConnection(self._options)
+                ServerConnection({}, conn) if conn else ServerConnection(self._options),
+                self._options,
             )
+
             if "password" in self._options:
                 self._options["password"] = None
             _add_session(new_session)
@@ -249,7 +302,9 @@ class Session:
     #: and create a :class:`Session` object.
     builder: SessionBuilder = SessionBuilder()
 
-    def __init__(self, conn: ServerConnection) -> None:
+    def __init__(
+        self, conn: ServerConnection, options: Optional[Dict[str, Any]] = None
+    ) -> None:
         if len(_active_sessions) >= 1 and is_in_stored_procedure():
             raise SnowparkClientExceptionMessages.DONT_CREATE_SESSION_IN_SP()
         self._conn = conn
@@ -278,13 +333,13 @@ class Session:
                 _PYTHON_SNOWPARK_USE_SCOPED_TEMP_OBJECTS_STRING, True
             )
         )
-
         self._file = FileOperation(self)
-
         self._analyzer = Analyzer(self)
         self._sql_simplifier_enabled: bool = self._get_client_side_session_parameter(
             _PYTHON_SNOWPARK_USE_SQL_SIMPLIFIER_STRING, True
         )
+        self._conf = self.RuntimeConfig(self, options or {})
+
         _logger.info("Snowpark Session information: %s", self._session_info)
 
     def __enter__(self):
@@ -326,6 +381,10 @@ class Session:
             finally:
                 _remove_session(self)
 
+    @property
+    def conf(self) -> RuntimeConfig:
+        return self._conf
+
     @property
     def sql_simplifier_enabled(self) -> bool:
         return self._sql_simplifier_enabled
@@ -1135,10 +1194,17 @@ class Session:
         supported sources (e.g. a file in a stage) as a DataFrame."""
         return DataFrameReader(self)
 
-    def _run_query(self, query: str, is_ddl_on_temp_object: bool = False) -> List[Any]:
-        return self._conn.run_query(query, is_ddl_on_temp_object=is_ddl_on_temp_object)[
-            "data"
-        ]
+    def _run_query(
+        self,
+        query: str,
+        is_ddl_on_temp_object: bool = False,
+        log_on_exception: bool = True,
+    ) -> List[Any]:
+        return self._conn.run_query(
+            query,
+            is_ddl_on_temp_object=is_ddl_on_temp_object,
+            log_on_exception=log_on_exception,
+        )["data"]
 
     def _get_result_attributes(self, query: str) -> List[Attribute]:
         return self._conn.get_result_attributes(query)
@@ -1169,7 +1235,7 @@ class Session:
         *,
         database: Optional[str] = None,
         schema: Optional[str] = None,
-        chunk_size: Optional[int] = None,
+        chunk_size: Optional[int] = WRITE_PANDAS_CHUNK_SIZE,
         compression: str = "gzip",
         on_error: str = "abort_statement",
         parallel: int = 4,
@@ -1188,7 +1254,8 @@ class Session:
             table_name: Name of the table we want to insert into.
             database: Database that the table is in. If not provided, the default one will be used.
             schema: Schema that the table is in. If not provided, the default one will be used.
-            chunk_size: Number of elements to be inserted once. If not provided, all elements will be dumped once.
+            chunk_size: Number of rows to be inserted once. If not provided, all rows will be dumped once.
+                Default to None normally, 100,000 if inside a stored procedure.
             compression: The compression used on the Parquet files: gzip or snappy. Gzip gives supposedly a
                 better compression, while snappy is faster. Use whichever is more appropriate.
             on_error: Action to take when COPY INTO statements fail. See details at
@@ -1390,7 +1457,7 @@ class Session:
                 schema=sf_schema,
                 quote_identifiers=True,
                 auto_create_table=True,
-                create_temp_table=True,
+                table_type="temporary",
             )
             set_api_call_source(t, "Session.create_dataframe[pandas]")
             return t
@@ -1767,12 +1834,18 @@ class Session:
         """
         return self._sp_registration
 
-    def call(self, sproc_name: str, *args: Any) -> Any:
+    def call(
+        self,
+        sproc_name: str,
+        *args: Any,
+        statement_params: Optional[Dict[str, Any]] = None,
+    ) -> Any:
         """Calls a stored procedure by name.
 
         Args:
             sproc_name: The name of stored procedure in Snowflake.
             args: Arguments should be basic Python types.
+            statement_params: Dictionary of statement level parameters to be set while executing this action.
 
         Example::
 
@@ -1793,16 +1866,9 @@ class Session:
             10
         """
         validate_object_name(sproc_name)
-
-        sql_args = []
-        for arg in args:
-            if isinstance(arg, Column):
-                sql_args.append(self._analyzer.analyze(arg._expression))
-            else:
-                sql_args.append(to_sql(arg, infer_type(arg)))
-        df = self.sql(f"CALL {sproc_name}({', '.join(sql_args)})")
+        df = self.sql(generate_call_python_sp_sql(self, sproc_name, *args))
         set_api_call_source(df, "Session.call")
-        return df.collect()[0][0]
+        return df.collect(statement_params=statement_params)[0][0]
 
     @deprecated(
         version="0.7.0",
